//+------------------------------------------------------------------+
//|                                      Export All Symbol Specs.mq5 |
//|                                       Copyright 2022, Dima Diall |
//+------------------------------------------------------------------+
#property copyright "Copyright 2022, Dima Diall"
#property version   "1.00"
#property script_show_inputs

#include <Trade/SymbolInfo.mqh>
#include <Files/File.mqh>

input string      CSV_Filename            = "SymbolSpecs/Market-Watch-Symbol-Specs.csv";
input string      CSV_Delimiter           = ";";
input bool        Only_MW_Symbols         = true;

void OnStart()
{
   CFile file;
   ushort first_char = StringGetCharacter(CSV_Delimiter, 0);
   ushort delim_char = first_char > 0 ? first_char : ';';
   if (!file.Open(CSV_Filename, FILE_CSV | FILE_WRITE, delim_char))
   {
      Alert("ABORTING! Error opening CSV file for writing: ", CSV_Filename);
      return;
   }

   CSymbolInfo symbol;
   int total_symbols = SymbolsTotal(Only_MW_Symbols);
   int counter = 0;

   PrintFormat("Found a total of %i symbols %s", total_symbols, Only_MW_Symbols ? "selected in Market Watch" : "in the broker's system");

   FileWrite(file.Handle(),
            "Name", "SelectedMW", "IsSynchronized", "Path", "Description",
            "Bank", "CurrencyBase", "CurrencyProfit", "CurrencyMargin",
            "Digits", "Point", "TickSize", "TickValue", "TickValueProfit", "TickValueLoss",
            "ContractSize", "LotsMin", "LotsMax", "LotsStep", "LotsLimit",
            "SwapLong", "SwapShort", "StopsLevel", "FreezeLevel",
            "Time", "Volume", "TicksBookDepth", "SpreadFloat", "Spread", "Bid", "Ask", "Last",
            "VolumeHigh", "VolumeLow", "BidHigh", "BidLow", "AskHigh", "AskLow", "LastHigh", "LastLow",
            "MarginInitial", "MarginMaintenance", "MarginHedgedUseLeg", "MarginHedged",
            "OrderMode", "TradeTimeFlags", "TradeFillFlags",
            "TradeCalcMode", "TradeMode", "TradeExecution", "SwapMode", "SwapRollover3days",
            "TradeCalcModeDescription", "TradeModeDescription", "TradeExecutionDescription",
            "SwapModeDescription", "SwapRollover3daysDescription",
            "FuturesStartTime", "FuturesExpirationTime"
   );

   for(int i = 0; i< total_symbols; i++)
   {
      string symbol_name = SymbolName(i, Only_MW_Symbols);
      if (!symbol.Name(symbol_name))
      {
         PrintFormat("Failed to select symbol  %s", symbol_name);
         continue;
      }

      symbol.Refresh();
      symbol.RefreshRates();
      FileWrite(file.Handle(),
               (string)    symbol.Name(),
               //--- fast access methods to the integer symbol propertyes
               (bool)      symbol.Select(),
               (bool)      symbol.IsSynchronized(),
               //--- fast access methods to the string symbol propertyes
               (string)    symbol.Path(),
               (string)    symbol.Description(),
               (string)    symbol.Bank(),
               (string)    symbol.CurrencyBase(),
               (string)    symbol.CurrencyProfit(),
               (string)    symbol.CurrencyMargin(),
               //--- tick parameters
               (int)       symbol.Digits(),
               (double)    symbol.Point(),
               (double)    symbol.TickSize(),
               (double)    symbol.TickValue(),
               (double)    symbol.TickValueProfit(),
               (double)    symbol.TickValueLoss(),
               //--- lots parameters
               (double)    symbol.ContractSize(),
               (double)    symbol.LotsMin(),
               (double)    symbol.LotsMax(),
               (double)    symbol.LotsStep(),
               (double)    symbol.LotsLimit(),
               //--- swaps
               (double)    symbol.SwapLong(),
               (double)    symbol.SwapShort(),
               //--- trade levels
               (int)       symbol.StopsLevel(),
               (int)       symbol.FreezeLevel(),
               //--- tick data
               (datetime)  symbol.Time(),
               (ulong)     symbol.Volume(),
               (int)       symbol.TicksBookDepth(),
               (bool)      symbol.SpreadFloat(),
               (int)       symbol.Spread(),
               (double)    symbol.Bid(),
               (double)    symbol.Ask(),
               (double)    symbol.Last(),
               //--- day highs & lows
               (ulong)     symbol.VolumeHigh(),
               (ulong)     symbol.VolumeLow(),
               (double)    symbol.BidHigh(),
               (double)    symbol.BidLow(),
               (double)    symbol.AskHigh(),
               (double)    symbol.AskLow(),
               (double)    symbol.LastHigh(),
               (double)    symbol.LastLow(),
               //--- margin parameters
               (double)    symbol.MarginInitial(),
               (double)    symbol.MarginMaintenance(),
               (bool)      symbol.MarginHedgedUseLeg(),
               (double)    symbol.MarginHedged(),
               //--- trade flags parameters
               (int)       symbol.OrderMode(),
               (int)       symbol.TradeTimeFlags(),
               (int)       symbol.TradeFillFlags(),
               //--- calculation, mode, execution & swap terms of trade
               EnumToString(symbol.TradeCalcMode()),
               EnumToString(symbol.TradeMode()),
               EnumToString(symbol.TradeExecution()),
               EnumToString(symbol.SwapMode()),
               EnumToString(symbol.SwapRollover3days()),
               //--- descriptions for calculation, mode, execution & swap terms of trade
               (string)    symbol.TradeCalcModeDescription(),
               (string)    symbol.TradeModeDescription(),
               (string)    symbol.TradeExecutionDescription(),
               (string)    symbol.SwapModeDescription(),
               (string)    symbol.SwapRollover3daysDescription(),
               //--- dates for futures
               (datetime)  symbol.StartTime(),
               (datetime)  symbol.ExpirationTime()
      );
      counter++;
   }
   file.Close();
   PrintFormat("Wrote %i symbol specs (out of %i exptected) to file %s", counter, total_symbols, CSV_Filename);
}
